/******************************************************************************
*******************************************************************************
**
**  Copyright (C) Sistina Software, Inc.  2002-2003  All rights reserved.
**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
**
**  This copyrighted material is made available to anyone wishing to use,
**  modify, copy, or redistribute it subject to the terms and conditions
**  of the GNU General Public License v.2.
**
*******************************************************************************
******************************************************************************/
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "gulm_defines.h"
#include "config_gulm.h"
#include "utils_ip.h"
#include "ccs.h"

/* Mostly, this is the extra functions I need to get stuff from the ccslib.
 * And a few wrapper functions so things work cleanly with it.
 */

/*****************************************************************************/
/* First data that is stored in the main. */

/* bits of data used by the log_*() and die() functions. */
extern uint32_t verbosity;
extern char *ProgramName;

/* confed things. */
extern gulm_config_t gulm_config;
extern char myName[256];

/*****************************************************************************/

/**
 * verify_name_and_ip_ccs - 
 * @name: 
 * @ip: 
 * 
 * check libresolv for ip match.
 * then check ccs for node {name {}} entry.
 *
 * 
 * Returns: =0:Deny =1:Allow
 */
int verify_name_and_ip_ccs(char *name, uint32_t ip)
{
   ccs_node_t *rt=NULL, *nd=NULL;
   char req[256];
   uint32_t testip;
   int n;

   if( get_ip_for_name_libc(name, &testip) != 0 ) {
      log_err("Failed to lookup ip for %s\n", name);
      goto fail;
   }

   if( testip != ip ) {
      log_err("For %s, ip %s doesn't match %s\n", name,
            iptostr(ip), iptostr(testip));
      goto fail;
   }

   if((n=open_ccs_file(&rt, "nodes.ccs")) != 0) {
      log_err("Failed to open nodes.ccs: %d\n", n);
      goto fail;
   }

   n = snprintf(req, 256, "nodes/%s", name);
   if( n < 0 || n > 255 ) {
      log_err("snprintf failed\n");
      goto fail; /* snprintf failed */
   }

   nd = find_ccs_node(rt, req, '/');
   if( nd == NULL ) {
      log_err("Node name %s is not in CCS. (correct case?)\n", name);
      goto fail;
   }

   close_ccs_file(rt);
   return 1;
fail:
   if( rt != NULL ) close_ccs_file(rt);
   return 0;
}

/**
 * verify_name_and_ip - 
 * @name: < name that this node is claiming.
 * @ip: < ip from which this node came
 * 
 * Just a switch board that calls the actual verify function based on what
 * the user chose in the config.
 * 
 * Returns: =0:Deny =1:Allow
 */
int verify_name_and_ip(char *name, uint32_t ip)
{
   switch(gulm_config.node_verification_type) {
      case nvt_tcpwrapper:
         return verify_name_and_ip_tcpwrapper(name, ip);
      case nvt_ccs:
         return verify_name_and_ip_ccs(name, ip);
      case nvt_allow:
         return 1;
      default:
         return 1;/* really should never happen, but you never know. */
   }
}

/* vim: set ai cin et sw=3 ts=3 : */
